<?php

/**
 * Get list of vocabularies (categories) together with node types
 *
 * @param array $node_types Array of strings with node types
 * @return string
 */
function _helper_get_vocabulary_list($node_types) {

  $vocabularies = taxonomy_get_vocabularies();
  $result = array();

  foreach ($vocabularies as $vocabulary) {
    $types = array();
    foreach ($vocabulary->nodes as $type) {
      $types[] = $node_types[$type];
    }

    $types_string = implode(', ', $types);
    $result[$vocabulary->vid] = check_plain($vocabulary->name);
    $result[$vocabulary->vid] .= (drupal_strlen($types_string)) ? ' (' . check_plain($types_string) . ')' : '';
  }

  return $result;
}

/**
 * Setup module config admin form
 */
function shoutem_api_admin_settings() {
  // List of available content types
  $node_types = array_map('check_plain', node_get_types('names'));

  $form['general'] = array(
    '#type' => 'fieldset',
    '#tree' => FALSE,
    '#title' => t('General settings'),
  );

  $form['general']['shoutem_api_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Content types exposed through ShoutEm API'),
    '#required' => TRUE,
    '#default_value' => variable_get('shoutem_api_node_types', array()),
    '#options' => $node_types,
    '#description' => t('Select content types available to ShoutEm clients via REST API.')
  );

  // Check if Taxonomy module is defined
  if (module_exists('taxonomy')) {
    $vocabs = _helper_get_vocabulary_list($node_types);
    $description = 'Selected vocabulary that best describes your content.';
    $description_no_cat = ' Currently, there are no categories defined. Please go to the <a href="@url">taxonomy administration</a> page and create at least one vocabulary.';

    $form['general']['shoutem_api_category_vid'] = array(
      '#type' => 'radios',
      '#title' => t('Choose content category'),
      '#required' => empty($vocabs) ? FALSE : TRUE,
      '#default_value' => variable_get('shoutem_api_category_vid', NULL),
      '#options' => $vocabs,
      '#description' => empty($vocabs) ? t($description_no_cat, array('@url' => url('admin/content/taxonomy'))) : t($description),
    );
  }
  else {
    $form['general']['shoutem_api_category_vid'] = array(
      '#prefix' => '<div>',
      '#value' => '<em>' . t('Note: ShoutEm module supports @module_name module that is not installed.', array('@module_name' => 'Taxonomy')) . '</em>',
      '#suffix' => '</div>'
    );
  }

  $form['imagefield'] = array(
        '#type' => 'fieldset',
        '#tree' => FALSE,
        '#title' => t('Images'),
        '#description' => t('Image support is provided using ImageField and ImageCache modules'),
  );

  /** ImageField support **/
  if (module_exists('imagefield') && module_exists('imagecache')) {
    $cckimagefieldsarray = array('' => t('No leading picture'));

    /*foreach (content_fields() as $ctype) {
      if ($ctype['widget']['module'] == 'imagefield') {
        $cckimagefieldsarray[$ctype['field_name']] = $ctype['field_name'];
      }
    }

    $form['imagefield']['shoutem_api_imagefield'] = array(
      '#type' => 'select',
      '#title' => t('ImageField for leading picture'),
      '#options' => $cckimagefieldsarray,
      '#default_value' => variable_get('shoutem_api_imagefield', ''),
      '#description' => t('Selected !field-name field will be used for node leading picture.', array('!field-name' => 'ImageField')),
    );*/
      $presets = array('' => t('None (Original)'));

      foreach (imagecache_presets() as $preset_id => $preset_info) {
        $preset = $preset_info['presetname'];
        $presets[$preset] = $preset;
      }

      $form['imagefield']['shoutem_api_imagecache_preset'] = array(
        '#type' => 'select',
        '#title' => t('ImageCache preset for images'),
        '#options' => $presets,
        '#default_value' => variable_get('shoutem_api_imagecache_preset', ''),
        '#description' => t('Selected !field-name preset will be used for node lead picture.', array('!field-name' => 'ImageCache')),
      );
  } // module_exists('imagefield')
  else {
    $form['imagefield']['shoutem_api_imagefield'] = array(
      '#type' => 'item',
      '#value' => '<em>' . t('Note: ShoutEm module supports @module_name modules that are not installed.', array('@module_name' => 'ImageField, ImageCache')) . '</em>',
    );
  }

  $form['overview'] = array(
    '#type' => 'fieldset',
    '#tree' => FALSE,
    '#title' => t('Comment configuration overview'),
    '#description' => t('An overview of the Comment module settings relevant for ShoutEm module'),
  );

  if (module_exists('comment')) {
    $form['overview'][] = array_merge(
        _helper_get_perm_table_item(),
        _helper_get_comments_overview_item($node_types)
    );
  }
  else {
    $form['overview'][] = array(
      '#type' => 'item',
      '#value' => '<em>' . t('Note: ShoutEm module supports @module_name module that is not installed.', array('@module_name' => 'Comment')) . '</em>',
    );
  }

  return system_settings_form($form);
}

/**
 * Generate table with permission overview for this module
 *
 * @return array Form API item
 */
function _helper_get_perm_table_item() {
  
  $result = db_query('SELECT r.rid, r.name, p.perm FROM {role} r LEFT JOIN {permission} p ON r.rid = p.rid ORDER BY rid');

  // Compile role array:
  // Add a comma at the end so when searching for a permission, we can
  // always search for "$perm," to make sure we do not confuse
  // permissions that are substrings of each other.
  while ($role = db_fetch_object($result)) {
    $role_permissions[$role->rid] = $role->perm . ',';
    $role_names[$role->rid] = $role->name;
  }

  $table_header = array_merge(array('Permission'), $role_names);

  $table_rows = array();
  foreach (shoutem_api_perm () as $perm) {
    $table_row = array(l($perm, 'admin/user/permissions', array('fragment' => 'module-shoutem_api')));
    foreach ($role_names as $rid => $name) {
      $table_row[] = check_plain($name) . ': ' . ((strpos($role_permissions[$rid], $perm . ',') !== FALSE) ? 'Yes' : 'No');
    }
    $table_rows[] = $table_row;
  }

  return array('shoutem_api_perms' => array(
    '#type' => 'item',
    '#title' => t('Permissions'),
    '#value' => theme('table', NULL, $table_rows),
    '#description' => t('Overview of ShoutEm specific permissions and their settings for each user role.'),
  ));
}

function _helper_get_comments_overview_item($node_types) {

  $comment_settings = array(
    t('Disabled'),
    t('Read only'),
    t('Read/Write')
  );

  $comment_setting_anonymous = array(
    COMMENT_ANONYMOUS_MAYNOT_CONTACT => t('Anonymous posters may not enter their contact information'),
    COMMENT_ANONYMOUS_MAY_CONTACT => t('Anonymous posters may leave their contact information'),
    COMMENT_ANONYMOUS_MUST_CONTACT => t('Anonymous posters must leave their contact information')
  );

  $enabled_node_types = variable_get('shoutem_api_node_types', array());

  $form = array();

  foreach ($node_types as $node_type => $node_type_name) {

    // Skip disabled content types
    if (empty($enabled_node_types[$node_type])) {
      continue;
    }

    $form['overview']['type' . $node_type] = array(
      '#type' => 'fieldset',
      '#tree' => FALSE,
      '#title' => t('@node-type comment settings', array('@node-type' => $node_type_name)),
    );

    $edit_link = ' [' . l('Edit', 'admin/content/node-type/' . $node_type) . ']';

    $form['overview']['type' . $node_type]['comment'] = array(
      '#prefix' => '<div class="form-item"><label style="display: inline">' . t('Commenting') . ': </label>',
      '#value' => $comment_settings[variable_get('comment_' . $node_type, COMMENT_NODE_READ_WRITE)] . $edit_link,
      '#suffix' => '<div class="description">' . t('Users with the <em>administer comments</em> permission will be able to override this setting.') . '</div></div>',
    );

    $form['overview']['type' . $node_type]['comment_anonymous'] = array(
      '#prefix' => '<div class="form-item"><label style="display: inline">' . t('Anonymous commenting') . ': </label>',
      '#value' => $comment_setting_anonymous[variable_get('comment_anonymous_'. $node_type, COMMENT_ANONYMOUS_MAYNOT_CONTACT)] . $edit_link,
      '#suffix' => '</div>'
    );

    if (!user_access('post comments via ShoutEm API', drupal_anonymous_user())) {
      unset($form['overview']['type' . $node_type]['comment_anonymous']['#value']);
    }

  }

  return $form;
}